package betterwithmods.common.blocks.mechanical.tile;

import betterwithmods.api.capabilities.CapabilityMechanicalPower;
import betterwithmods.api.tile.ICrankable;
import betterwithmods.api.tile.IMechanicalPower;
import betterwithmods.common.BWSounds;
import betterwithmods.common.blocks.mechanical.BlockMechMachines;
import betterwithmods.common.blocks.tile.TileBasicInventory;
import betterwithmods.common.registry.bulk.manager.MillManager;
import betterwithmods.common.registry.bulk.recipes.MillRecipe;
import betterwithmods.util.InvUtils;
import betterwithmods.util.MechanicalUtil;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.ITickable;
import net.minecraft.util.NonNullList;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.Capability;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

public class TileEntityMill extends TileBasicInventory implements ITickable, IMechanicalPower, ICrankable {

    public static final int GRIND_TIME = 200;

    public int power;
    public int grindCounter;
    private int grindType = 0;
    private boolean validateContents;
    private boolean containsIngredientsToGrind;

    public TileEntityMill() {
        this.grindCounter = 0;
        this.validateContents = true;
    }

    public boolean isActive() {
        return power > 0;
    }

    @Override
    public boolean shouldRefresh(World world, BlockPos pos, IBlockState oldState, IBlockState newState) {
        return oldState.func_177230_c() != newState.func_177230_c();
    }

    public BlockMechMachines getBlock() {
        return (BlockMechMachines) this.func_145838_q();
    }

    @Override
    public void func_73660_a() {
        if (this.getBlockWorld().field_72995_K)
            return;

        this.power = calculateInput();

        getBlock().setActive(field_145850_b, field_174879_c, isActive());

        if (this.validateContents)
            validateContents();

        if (isActive())
            if (getBlockWorld().field_73012_v.nextInt(20) == 0)
                getBlockWorld().func_184133_a(null, field_174879_c, BWSounds.STONEGRIND, SoundCategory.BLOCKS, 0.5F + getBlockWorld().field_73012_v.nextFloat() * 0.1F, 0.5F + getBlockWorld().field_73012_v.nextFloat() * 0.1F);

        if (this.containsIngredientsToGrind && isActive()) {
            if (!this.getBlockWorld().field_72995_K) {
                if (grindType == 2) {
                    if (this.getBlockWorld().field_73012_v.nextInt(25) < 2) {
                        getBlockWorld().func_184133_a(null, field_174879_c, SoundEvents.field_187555_bJ, SoundCategory.BLOCKS, 1F, getBlockWorld().field_73012_v.nextFloat() * 0.4F + 0.8F);
                    }
                } else if (grindType == 1) {
                    if (this.getBlockWorld().field_73012_v.nextInt(20) < 2)
                        getBlockWorld().func_184133_a(null, field_174879_c, SoundEvents.field_187863_gH, SoundCategory.BLOCKS, 2.0F, (getBlockWorld().field_73012_v.nextFloat() - getBlockWorld().field_73012_v.nextFloat()) * 0.2F + 1.0F);
                }
            }
            this.grindCounter++;
            if (this.grindCounter > GRIND_TIME - 1) {
                grindContents();
                this.grindCounter = 0;
                this.validateContents = true;
            }
        }
    }

    @Override
    public void func_145839_a(NBTTagCompound tag) {
        super.func_145839_a(tag);
        if (tag.func_74764_b("GrindCounter"))
            this.grindCounter = tag.func_74762_e("GrindCounter");
        this.power = tag.func_74762_e("power");
    }

    @Override
    public NBTTagCompound func_189515_b(NBTTagCompound tag) {
        super.func_189515_b(tag);
        tag.func_74768_a("GrindCounter", this.grindCounter);
        tag.func_74768_a("power", power);
        return tag;
    }

    @Override
    public int getInventorySize() {
        return 3;
    }

    public int getGrindType() {
        return this.grindType;
    }

    @Override
    public void func_70296_d() {
        super.func_70296_d();
        validateContents();
        if (this.getBlockWorld() != null && !this.getBlockWorld().field_72995_K) {
            if (grindType == 1)
                this.getBlockWorld().func_184133_a(null, this.field_174879_c, SoundEvents.field_187871_gL, SoundCategory.BLOCKS, 1F, 1.0F);
            this.validateContents = true;
        }
    }

    private void ejectStack(ItemStack stack) {
        List<EnumFacing> validDirections = new ArrayList<>();
        for (EnumFacing facing : EnumFacing.field_176754_o) {
            IBlockState check = getBlockWorld().func_180495_p(field_174879_c.func_177972_a(facing));
            if (check.func_177230_c().func_176200_f(getBlockWorld(), field_174879_c.func_177972_a(facing)) || getBlockWorld().func_175623_d(field_174879_c.func_177972_a(facing)))
                validDirections.add(facing);
        }

        if (validDirections.isEmpty()) {
            IBlockState down = getBlockWorld().func_180495_p(field_174879_c.func_177972_a(EnumFacing.DOWN));
            if (down.func_177230_c().func_176200_f(getBlockWorld(), field_174879_c.func_177972_a(EnumFacing.DOWN)) || getBlockWorld().func_175623_d(field_174879_c.func_177972_a(EnumFacing.DOWN)))
                validDirections.add(EnumFacing.DOWN);
        }

        BlockPos offset;
        if (validDirections.size() > 1)
            offset = field_174879_c.func_177972_a(validDirections.get(getBlockWorld().field_73012_v.nextInt(validDirections.size())));
        else if (validDirections.isEmpty())
            offset = field_174879_c.func_177972_a(EnumFacing.UP);
        else
            offset = field_174879_c.func_177972_a(validDirections.get(0));

        InvUtils.ejectStackWithOffset(getBlockWorld(), offset, stack);
    }

    public double getGrindProgress() {
        return this.grindCounter / (double) GRIND_TIME;
    }

    public boolean isGrinding() {
        return this.grindCounter > 0;
    }

    private boolean grindContents() {
        MillManager mill = MillManager.getInstance();
        List<Object> ingredients = mill.getValidCraftingIngredients(inventory);

        if (ingredients != null) {
            if (grindType == 1)
                this.getBlockWorld().func_184133_a(null, field_174879_c, SoundEvents.field_187859_gF, SoundCategory.BLOCKS, 1.0F, 1.0F);
            NonNullList<ItemStack> output = mill.craftItem(field_145850_b, this, inventory);
            if (!output.isEmpty()) {
                for (ItemStack anOutput : output) {
                    ItemStack stack = anOutput.func_77946_l();
                    if (!stack.func_190926_b())
                        ejectStack(stack);
                }
            }
            return true;
        }
        return false;
    }

    private void validateContents() {
        int oldGrindType = getGrindType();
        int newGrindType = 0;
        MillRecipe recipe = MillManager.getInstance().getMostValidRecipe(inventory);
        if (recipe != null) {
            this.containsIngredientsToGrind = true;
            newGrindType = recipe.getGrindType();
        } else {
            this.grindCounter = 0;
            this.containsIngredientsToGrind = false;
        }
        this.validateContents = false;
        if (oldGrindType != newGrindType) {
            this.grindType = newGrindType;
        }
    }

    @Override
    public int getMechanicalOutput(EnumFacing facing) {
        return -1;
    }

    @Override
    public int getMechanicalInput(EnumFacing facing) {
        if (facing.func_176740_k().func_176720_b())
            return MechanicalUtil.getPowerOutput(field_145850_b, field_174879_c.func_177972_a(facing), facing.func_176734_d());
        if (field_145850_b.func_175625_s(field_174879_c.func_177972_a(facing)) instanceof TileCrank) {
            return MechanicalUtil.getPowerOutput(field_145850_b, field_174879_c.func_177972_a(facing), facing.func_176734_d());
        }
        return 0;
    }

    @Override
    public int getMaximumInput(EnumFacing facing) {
        return 1;
    }

    @Override
    public int getMinimumInput(EnumFacing facing) {
        return 0;
    }

    @Override
    public boolean hasCapability(@Nonnull Capability<?> capability, @Nonnull EnumFacing facing) {
        if (capability == CapabilityMechanicalPower.MECHANICAL_POWER)
            return true;
        return super.hasCapability(capability, facing);
    }

    @Nonnull
    @Override
    public <T> T getCapability(@Nonnull Capability<T> capability, @Nonnull EnumFacing facing) {
        if (capability == CapabilityMechanicalPower.MECHANICAL_POWER)
            return CapabilityMechanicalPower.MECHANICAL_POWER.cast(this);
        return super.getCapability(capability, facing);
    }

    @Override
    public World getBlockWorld() {
        return super.func_145831_w();
    }

    @Override
    public BlockPos getBlockPos() {
        return func_174877_v();
    }

    public boolean isUseableByPlayer(EntityPlayer player) {
        return this.getBlockWorld().func_175625_s(this.field_174879_c) == this && player.func_70092_e(this.field_174879_c.func_177958_n() + 0.5D, this.field_174879_c.func_177956_o() + 0.5D, this.field_174879_c.func_177952_p() + 0.5D) <= 64.0D;
    }
}
